/******************************************************************************
 *
 * Easy-sql, version 1.0.5
 * Copyright (c) 1997-1998 Andrew Maltsev <am@amsoft.ru>
 * See License!
 *
 * Prototypes of easysql classes and methods.
 *
 ******************************************************************************
 * $Id: easysql.h.proto,v 1.12 1998/04/17 13:41:25 am Exp $
*/
#ifndef __AM_easysql_h
#define __AM_easysql_h

#include <stdio.h>		/* For NULL declaration */
#include <stdarg.h>		/* For va_list declaration */

#define	EASYSQL_VERSION	"1.0.5"
#define EASYSQL_LOCATION "/usr/local/easysql"

/* To help GCC track format problems
*/
#ifdef __GNUC__
#define	attribute(x) __attribute__((x))
#else
#define	attribute(x) /* x */
#endif

/* Data types
*/
typedef void *esqlResult;
typedef char const * const *esqlRow;

/* C++ definitions. If you are looking for plain C portion - skip up to `_C_' label
*/
#ifdef __cplusplus
///////////////////////////////////////////////////////////////////////////////
// Support classes
//
class easysql_driver_shell;
class easysql_arguments;
class easysql_queries_register;

///////////////////////////////////////////////////////////////////////////////
// Supporting base class for setting and reporting error messages
//
class easysql_error_reporting
{ char *category;
  char *text;
  unsigned size;
  unsigned level;
 protected:
  easysql_error_reporting(char const *name=NULL, unsigned lvl=0);
  ~easysql_error_reporting();
  void set_level(unsigned);
 public:
  char const *error_text();
  void no_error();
  void set_error(char const *fmt, ...) attribute(format(printf,2,3));
};

///////////////////////////////////////////////////////////////////////////////
// Main SQL interface class
//
class easysql : public easysql_error_reporting
{ easysql_driver_shell *driver;
  easysql_queries_register *qreg;
  char *drvname;
  char const *to_be_escaped;
 protected:
  easysql_arguments *args;
  unsigned retry_delay;
 public:
  easysql(char const *type);		// Loading driver
  ~easysql();				// Unloading driver
  operator bool();			// Driver and other data ok?

// Text of last error (overrides easysql_error_reporting::error_text)
//
  char const *error_text();

// Most usable functions (interface level 1)
//
  esqlResult query(char const *q, ...) attribute(format(printf,2,3));
  esqlRow row(esqlResult r);
  bool rfree(esqlResult r);
  char const *string(char const *str, char *buf=NULL);	// abc'"\def  ->  abc\'\"\\def

// Sometimes usable functions (interface level 2)
//
  void reset();
  bool option(char const *opt);		// Arguments to be given to driver
  bool option(char const *name,		// when connecting to engine.
              char const *value);
  bool driver_ok();			// Driver successfully loaded?
  char const *driver_status();		// Free style driver status info
  char const *driver_version();		// `name version api engine'
  unsigned set_retry_delay(unsigned d);	// Setting retry time for failed queries

// Not really needed functions. Just in case of.. (interface level 3)
//
  bool connection_ok();
  bool connect();
  bool drop();
  esqlResult _query(char const *q, va_list ap) attribute(format(printf,2,0));
};
#endif		/* __cplusplus */

/* ___________________________________ _C_ ____________________________________
 *
 * Some stubs, to use in non-C++ applications. Less suitable, but still usable.
*/
typedef void *esqlDB;
#ifdef __cplusplus
extern "C" {
#endif
esqlDB easysql_open(char const *dbname);
int easysql_close(esqlDB db);
char const *easysql_error_text(esqlDB db);
esqlResult easysql_query(esqlDB db, char const *q, ...) attribute(format(printf,2,3));
esqlResult easysql_query_ap(esqlDB db, char const *query, va_list ap) attribute(format(printf,2,0));
esqlRow easysql_row(esqlDB db, esqlResult r);
int easysql_rfree(esqlDB db, esqlResult r);
void easysql_reset(esqlDB db);
int easysql_put_option(esqlDB db, char const *opt);
int easysql_set_option(esqlDB db, char const *name, char const *value);
int easysql_driver_ok(esqlDB db);
char const *easysql_driver_status(esqlDB db);
char const *easysql_driver_version(esqlDB db);
int easysql_ok(esqlDB db);
unsigned easysql_set_retry_delay(esqlDB db, unsigned d);
int easysql_connection_ok(esqlDB db);
int easysql_connect(esqlDB db);
int easysql_drop(esqlDB db);
char const *easysql_string(esqlDB db, char const *str, char *buf);
#ifdef __cplusplus
}
#endif

/* _______________________________Macroses_____________________________________
 * Definition of drivers we want to link statically
*/
#define FastDriver(drv)	extern void *drv##_register_entry;	\
                        static void const * drv##_use_driver=drv##_register_entry

/* ________________________________Debug_______________________________________
 * If you want to have more internal debug messages printed..
*/
#define EASYSQL_D_L1	0x0001	/* Levels of debug		*/
#define EASYSQL_D_L2	0x0002	/* L1 - only high priority msgs	*/
#define	EASYSQL_D_L3	0x0003	/* L4 - all messages		*/
#define EASYSQL_D_L4	0x0004
/**/
#define EASYSQL_D_MISC	0x0010	/* Miscelaneous			*/
#define EASYSQL_D_SQL	0x0020	/* SQL commands, results..	*/
#define EASYSQL_D_DRV	0x0040	/* Driver loading/registration	*/
#define	EASYSQL_D_IDRV	0x0080	/* Internal driver's info	*/
#define	EASYSQL_D_C	0x0100	/* C-stubs info                 */
/**/
#define EASYSQL_D_ALL	0x7fff	/* All possible debug info	*/
#define EASYSQL_D_NONE	0x0000	/* Nothing			*/
#define EASYSQL_D_ERROR	0x8000	/* Only errors (by default)	*/
/**/
#ifdef __cplusplus
extern "C" {
#endif
void easysql_set_log_level(unsigned l);
void easysql_set_log_flush(int fl);
unsigned easysql_get_log_level();
int easysql_set_log_file(char const *name);
char const *easysql_get_log_file();
void easysql_reopen_log();
#ifdef __cplusplus
}
#endif

#undef attribute
#endif	// __AM_amsql_h
